<?php

namespace Drupal\content_export_yaml;


use Symfony\Component\Yaml\Dumper;
use Symfony\Component\Yaml\Yaml as SymfonyYaml;
use Symfony\Component\Yaml\Parser;
use Symfony\Component\HttpFoundation\RedirectResponse;
/**
 * Created by PhpStorm.
 * User: USER
 * Date: 11/13/18
 * Time: 2:04 PM
 */
class ContentExport {

  public $logger;
  public $db;

  public function __construct() {
    $this->logger = \Drupal::logger('content_export_yaml');
    $this->db = new DBManager();
  }
  function content_type_list() {
    $node_types = \Drupal\node\Entity\NodeType::loadMultiple();
    $options = [];
    foreach ($node_types as $node_type) {
      $options[$node_type->id()] = $node_type->label();
    }
    return $options;
  }

  function load_entity_list($entity, $id_label, $bundle_label, $bundle, $ranges_nid = []) {
    $factory = \Drupal::entityTypeManager()->getStorage($entity)->getQuery();
    if ($bundle != "all") {
      $factory->condition($bundle_label, $bundle);
    }
    if (!empty($ranges_nid)) {
      $factory->condition($id_label, $ranges_nid, 'BETWEEN');

    }
    return $factory->execute();
  }

  function load_block_list($bundle, $ranges_nid = []) {
    $factory = \Drupal::entityTypeManager()
      ->getStorage("block_content")
      ->getQuery();
    if ($bundle != "all") {
      $factory->condition('type', $bundle);
    }
    if (!empty($ranges_nid)) {
      $factory->condition('id', $ranges_nid, 'BETWEEN');
    }
    return $factory->execute();
  }

  function load_term_list($bundle, $ranges_nid = []) {
    $factory = \Drupal::entityTypeManager()
      ->getStorage("taxonomy_term")
      ->getQuery();
    if ($bundle != "all") {
      $factory->condition('vid', $bundle);
    }
    if (!empty($ranges_nid)) {
      $factory->condition('tid', $ranges_nid, 'BETWEEN');
    }
    return $factory->execute();
  }

  function load_term_config_list($bundle, $ranges_nid = []) {
    $items = [];
    $config = \Drupal::config('content_export_yaml.contentexportsetting');
    $themes_str = $config->get('path_export_content_folder');
    if ($themes_str) {
      if (empty($ranges_nid)) {
        $items = $this->readDirectory($themes_str . "/node/" . $bundle);
        foreach ($items as $key => $file) {
          if (file_exists($file)) {
            $items[$key] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
      else {
        for ($i = $ranges_nid[0]; $i < $ranges_nid[0] + 1; $i++) {
          $file = DRUPAL_ROOT . '/' . $themes_str . "/node/" . $bundle . "/" . $i . ".yml";
          if (file_exists($file)) {
            $items[$i] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
    }
    else {
      $this->logger->error('Path directory empty ');
    }
    return $items;
  }

  function load_node_config_list($bundle, $ranges_nid = []) {
    $items = [];
    $config = \Drupal::config('content_export_yaml.contentexportsetting');
    $themes_str = $config->get('path_export_content_folder');
    if ($themes_str) {
      if (empty($ranges_nid)) {
        $items = $this->readDirectory($themes_str . "/node/" . $bundle);
        foreach ($items as $key => $file) {
          if (file_exists($file)) {
            $items[$key] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
      else {
        for ($i = $ranges_nid[0]; $i < $ranges_nid[0] + 1; $i++) {
          $file = DRUPAL_ROOT . '/' . $themes_str . "/node/" . $bundle . "/" . $i . ".yml";
          if (file_exists($file)) {
            $items[$i] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
    }
    else {
      $this->logger->error('Path directory empty ');
    }
    return $items;
  }

  function load_block_list_config($bundle, $ranges_nid = []) {
    if ($bundle == 'all') {
      $bundle = NULL;
    }
    return $this->load_entity_config_list('block_content', $bundle, $ranges_nid);
  }

  function load_entity_config_list_with_path($entity, $bundle, $ranges_nid = [], $path = NULL) {
    $items = [];
    if ($path) {
      $themes_str = $path;
    }
    else {
      $config = \Drupal::config('content_export_yaml.contentexportsetting');
      $themes_str = $config->get('path_export_content_folder');
    }

    if ($themes_str) {
      if (empty($ranges_nid)) {
        if ($bundle) {
          $items = $this->readDirectory($themes_str . "/" . $entity . "/" . $bundle);
        }
        else {
          $items = $this->readDirectory($themes_str . "/" . $entity);
        }
        foreach ($items as $key => $file) {
          if (file_exists($file)) {
            $items[$key] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
      else {
        for ($i = $ranges_nid[0]; $i < $ranges_nid[0] + 1; $i++) {
          if ($bundle) {
            $file = DRUPAL_ROOT . '/' . $themes_str . "/" . $entity . "/" . $bundle . "/" . $i . ".yml";
          }
          else {
            $file = DRUPAL_ROOT . '/' . $themes_str . "/" . $entity . "/" . $i . ".yml";
          }
          if (file_exists($file)) {
            $items[$i] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
    }
    else {
      $this->logger->error('Path directory empty ');
    }
    return $items;
  }

  function load_entity_config_list($entity, $bundle = NULL, $ranges_nid = []) {
    $items = [];
    $config = \Drupal::config('content_export_yaml.contentexportsetting');
    $themes_str = $config->get('path_export_content_folder');
    if ($themes_str) {
      if (empty($ranges_nid)) {
        if ($bundle) {
          $items = $this->readDirectory($themes_str . "/" . $entity . "/" . $bundle);
        }
        else {
          $items = $this->readDirectory($themes_str . "/" . $entity);
        }
        foreach ($items as $key => $file) {
          if (file_exists($file)) {
            $items[$key] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
      else {
        for ($i = $ranges_nid[0]; $i < $ranges_nid[0] + 1; $i++) {
          if ($bundle) {
            $file = DRUPAL_ROOT . '/' . $themes_str . "/" . $entity . "/" . $bundle . "/" . $i . ".yml";
          }
          else {
            $file = DRUPAL_ROOT . '/' . $themes_str . "/" . $entity . "/" . $i . ".yml";
          }
          if (file_exists($file)) {
            $items[$i] = file_get_contents($file, FILE_USE_INCLUDE_PATH);
          }
          else {
            $this->logger->error('File  not find exist : ' . $file);
          }
        }
      }
    }
    else {
      $this->logger->error('Path directory empty ');
    }
    return $items;
  }

  function readDirectory($directory) {
    $path_file = [];
    if (is_dir(DRUPAL_ROOT . $directory)) {
      $it = scandir(DRUPAL_ROOT . $directory);
      if (!empty($it)) {
        foreach ($it as $fileinfo) {
          if ($fileinfo && strpos($fileinfo, '.yml') !== FALSE) {
            $file = DRUPAL_ROOT . $directory . "/" . $fileinfo;
            if (file_exists($file)) {
              $path_file[] = DRUPAL_ROOT . $directory . "/" . $fileinfo;
            }
          }
        }
      }
    }
    return $path_file;
  }

  function load_node_list($bundle, $ranges_nid = []) {
    $db = \Drupal::database();
    $query = $db->select('node_field_data', 'n');
    $query->fields('n', ['nid']);
    $query->condition('n.type', $bundle);
    if (!empty($ranges_nid)) {
      $query->condition('n.nid', $ranges_nid, 'BETWEEN');
    }
    return $query->execute()->fetchAllAssoc('nid');
  }

  function import($id, $entity) {
    $parsed = new Parser();
    $node_object = $parsed->parse($id, SymfonyYaml::PARSE_OBJECT);
    if (is_object($node_object)) {

      switch ($entity) {
        case "node":
          return $this->saving($node_object);
          break;
        case "block_content":
          return $this->saving_block($node_object);
          break;
      }
    }
    else {
      $this->logger->error('Failed to save item');
    }
    return FALSE;

  }

  function importEntity($yaml_object, $entity) {
    $bundle_label = \Drupal::entityTypeManager()
      ->getDefinition($entity)
      ->getKey('bundle');
    $id_label = \Drupal::entityTypeManager()
      ->getDefinition($entity)
      ->getKey('id');
    $key_label = \Drupal::entityTypeManager()
      ->getDefinition($entity)
      ->getKey('label');
    $parsed = new Parser();
    $node_object = $parsed->parse($yaml_object, SymfonyYaml::PARSE_OBJECT);
    if (is_object($node_object)) {
      return $this->savingEntity($node_object, $entity, $id_label, $bundle_label, $key_label);
    }
    else {
      $this->logger->error('Failed to save item');
    }
    return FALSE;

  }

  function importContentNode($item) {
    $parsed = new Parser();
    $node_object = $parsed->parse($item, SymfonyYaml::PARSE_OBJECT);
    if (is_object($node_object)) {
      return $this->saving($node_object);
    }
    else {
      $this->logger->error('Failed to save item');
    }
  }
  function is_exist_node($id,$bundle,$entity){
    $bundle_label = \Drupal::entityTypeManager()->getDefinition($entity)->getKey('bundle');
    $id_label = \Drupal::entityTypeManager()->getDefinition($entity)->getKey('id');
    return \Drupal::entityTypeManager()
      ->getStorage($entity)
      ->loadByProperties([
        $id_label => $id,
        $bundle_label => $bundle
      ]);
  }
  function savingEntity($enity_clone, $entity, $id_label, $bundle_label, $key_label = NULL) {

    if ($key_label) {
      $entity_list = \Drupal::entityTypeManager()
        ->getStorage($entity)
        ->loadByProperties([
          $id_label => $enity_clone->id(),
          $bundle_label => $enity_clone->bundle(),
          $key_label => $enity_clone->label(),
        ]);
    }
    else {
      $entity_list = \Drupal::entityTypeManager()
        ->getStorage($entity)
        ->loadByProperties([
          $id_label => $enity_clone->id(),
          $bundle_label => $enity_clone->bundle(),
        ]);
    }
    if (!empty($entity_list)) {
      $status = $enity_clone->save();
      if ($status == 2) {
        drupal_set_message(t($bundle_label . " with " . $id_label . "=" . $enity_clone->id() . " update "));
      }
    }
    else {

      $enity_clone->{$id_label} = NULL;
      // Also handle modules that attach a UUID to the node.
      $enity_clone->uuid = \Drupal::service('uuid')->generate();
      // Anyonmymous users don't have a name.
      $enity_clone->created = time();
      $enity_clone->uid = 0;
      $status = $enity_clone->save();
      if ($status == 1) {
        drupal_set_message(t("New Item of " . $bundle_label . " was created "));

      }

    }
    return $status;
  }

  function saving($node_clone) {

    $node_list = \Drupal::entityTypeManager()
      ->getStorage('node')
      ->loadByProperties([
        'nid' => $node_clone->id(),
        'type' => $node_clone->getType(),
      ]);
    if (!empty($node_list)) {
      print ("update ");
      return $node_clone->save();
    }
    else {
      print ("insert ");
      $node_clone->nid = NULL;
      $node_clone->vid = NULL;
      $node_clone->tnid = NULL;
      $node_clone->log = NULL;
      // Also handle modules that attach a UUID to the node.
      $node_clone->uuid = \Drupal::service('uuid')->generate();
      $node_clone->vuuid = NULL;
      // Anyonmymous users don't have a name.
      $node_clone->created = time();
      $node_clone->path = NULL;
      $node_clone->uid = 0;
      return $node_clone->save();
    }
  }

  function saving_block($block) {

    $list = \Drupal::entityTypeManager()
      ->getStorage('block_content')
      ->loadByProperties([
        'id' => $block->id(),
        'type' => $block->bundle(),
      ]);
    if (!empty($list)) {
      print ("update ");
      return $block->save();
    }
    else {
      print ("insert ");
      $block->id = NULL;
      // Also handle modules that attach a UUID to the node.
      $block->uuid = \Drupal::service('uuid')->generate();
      $block->created = time();
      $block->uid = 0;
      $status = $block->save();
      return $status;
    }
  }

  function export($id, $entity, $type = NULL) {
    $config = \Drupal::config('content_export_yaml.contentexportsetting');
    $themes_str = $config->get('path_export_content_folder');
    return $this->exportBase($id, $entity,$themes_str);

  }
  function exportBase($id, $entity, $themes_str_path){
    if (is_object($id)) {
      $item = $id;
    }
    else {
      $item = \Drupal::entityTypeManager()
        ->getStorage($entity)->load($id);
    }
    if (is_object($item)) {
      $yaml_content = $this->parserYAMLObject($item);
      if ($themes_str_path) {
        $themes_str = DRUPAL_ROOT . '/' . $themes_str_path;
        //        if ($type) {
        //          $final_path = $themes_str . '/' . $entity . '/' . $type;
        //        }
        //        else {
        $final_path = $themes_str . '/' . $entity.'/'.$item->bundle();
        //      }
        $status = $this->generateFile($final_path, $item->id(), $yaml_content);
        if($status){
          $dbstatus = $this->db->is_exist($item->id(),$entity,$item->bundle());
          if(empty($dbstatus)){
            $fields = array(
              'entity_id'=> $item->id() ,
              'entity_type' => $entity,
              'bundle' => $item->bundle(),
              'label' => $item->label(),
              'file' => $themes_str_path.'/'.$entity.'/'.$item->bundle().'/'.$item->id().'.yml'
            );
            $this->db->insert($fields);
          }
        }
        return $status ;
      }
      else {
        $this->logger->error('Path directory empty ');
        drupal_set_message(t('Path directory empty '), 'error');

        return FALSE;
      }
    }
    return FALSE;
  }
  function exportWithPath($id, $entity, $options = []) {
    $config = \Drupal::config('content_export_yaml.contentexportsetting');
    $path_config = $config->get('path_export_content_folder');
    $themes_str_path = isset($options["path"]) ? $options["path"] : $path_config;
    return $this->exportBase($id, $entity,$themes_str_path);

  }

  /**for node **/
  function exportContent($nid) {
    $item = \Drupal::entityTypeManager()
      ->getStorage('node')->load($nid);
    return $this->export($nid, 'node', $item->getType());
  }

  function parserYAMLObject($node) {
    $yaml = new Dumper(2);
    return $yaml->dump($node, PHP_INT_MAX, 0, SymfonyYaml::DUMP_OBJECT);
  }

  function generateFile($directory, $filename, $content) {
    $fileSystem = \Drupal::service('file_system');
    if (!is_dir($directory)) {
      if ($fileSystem->mkdir($directory, 0777, TRUE) === FALSE) {
        $this->logger->error('Failed to create directory ' . $directory);
        drupal_set_message(t('Failed to create directory ' . $directory), 'error');
        return FALSE;
      }
    }
    if (file_put_contents($directory . '/' . $filename . '.yml', $content) === FALSE) {
      drupal_set_message(t('Failed to write file ' . $filename), 'error');
      $this->logger->error('Failed to write file ' . $filename);
      return FALSE;
    }
    if (@chmod($directory . '/' . $filename . '.html.twig', 0777)) {
      drupal_set_message(t('Failed to change permission file ' . $filename), 'error');
      $this->logger->error('Failed to change permission file ' . $filename);
    }
    return TRUE;
  }

  public function download_yml($yml) {
    $path_file = \Drupal::service('file_system')->realpath("public://temp_yml");
    $file_name = "download";
    $this->delete($path_file . "/" . $file_name . ".yml");
    $status = $this->yml_copy($file_name, $yml, $path_file);
    if ($status) {
      $file_temp = "/sites/default/files/temp_yml/" . $file_name . ".yml";
      @chmod($file_temp, 0777);
      return $file_temp;
    }
    else {
      drupal_set_message("failed to download", "error");
    }
  }

  function yml_copy($file_name, $file, $path_export) {
    $file_full_path = DRUPAL_ROOT . $file;
    $fileSystem = \Drupal::service('file_system');
    if (!is_dir($path_export)) {
      if ($fileSystem->mkdir($path_export, 0777, TRUE) === FALSE) {
        $this->logger->error('Failed to create directory ' . $path_export);
        return FALSE;
      }
    }
    if (!copy($file_full_path, $path_export . "/" . $file_name . ".yml")) {
      drupal_set_message("failed to copy $file", "error");
      return FALSE;
    }
    else {
      drupal_set_message("Upload Success", "status");
      @chmod($path_export . "/" . $file_name . ".yml", 0777);
      return TRUE;
    }
  }


  function importByFilePath($file) {
    $parsed = new Parser();
    $path_file = DRUPAL_ROOT . $file;
    $status = 0;
    if (file_exists($path_file)) {
      $item_yaml = file_get_contents($path_file, FILE_USE_INCLUDE_PATH);
      $item_object = $parsed->parse($item_yaml, SymfonyYaml::PARSE_OBJECT);
      if (is_object($item_object)) {
        $new_item = $item_object->createDuplicate();

        $status = $new_item->save();
        if ($status == 1) {
          $entity_type = $new_item->getEntityTypeId();
          $type = $new_item->bundle();
          $id = $new_item->id();
          $this->export($id, $entity_type, $type);
          $this->delete($file);
        }
      }
    }
    return $status;
  }

  function delete($file) {
    $file = DRUPAL_ROOT . $file;
    if (file_exists($file)) {
      if (is_writable($file) && @unlink($file)) {
        return TRUE;
      }
      else {
        $this->logger->error('File  not write : ' . $file);
        drupal_set_message('File  not write : ' . $file, 'error');

        return FALSE;
      }
    }
    return FALSE;
  }

  function listFolderFiles($dir) {
    $fileInfo = scandir($dir);
    $allFileLists = [];
    $parsed = new Parser();

    foreach ($fileInfo as $folder) {


      if ($folder !== '.' && $folder !== '..') {
        if (is_dir($dir . DIRECTORY_SEPARATOR . $folder) === TRUE) {

          $allFileLists[$folder] = $this->listFolderFiles($dir . DIRECTORY_SEPARATOR . $folder);
        }
        else {
          $path_file = $dir . DIRECTORY_SEPARATOR . $folder;
          $ext = pathinfo($path_file, PATHINFO_EXTENSION);
          if (file_exists($path_file) && $ext == 'yml') {
            $item_yaml = file_get_contents($path_file, FILE_USE_INCLUDE_PATH);
            if ($item_yaml) {
              try {
                $item_object = \Symfony\Component\Yaml\Yaml::parse($item_yaml, SymfonyYaml::PARSE_OBJECT);
              } catch (Exception $e) {
                drupal_set_message("'Message: " . $e->getMessage(), "error");

              }
              if ($item_object && is_object($item_object)) {
                $path = str_replace(DRUPAL_ROOT, "", $path_file);
                $allFileLists[$folder] = [
                  "file" => $folder,
                  "path" => $path,
                  "entity" => $item_object,
                ];
              }
            }
          }
        }
      }
    }

    return $allFileLists;
  }//end listFolderFiles()

  function load_exported_all() {

    $config = \Drupal::config('content_export_yaml.contentexportsetting');
    $themes_str = $config->get('path_export_content_folder');
    $items = [];
    if ($themes_str) {
      $result = $this->listFolderFiles(DRUPAL_ROOT . $themes_str);
      foreach ($result as $key => $item_entity_type) {
        foreach ($item_entity_type as $key => $item_bundle) {
          foreach ($item_bundle as $key => $item) {
            $items[] = $item;
            $entity_object = $item['entity'] ;
            $entity = $entity_object->getEntityTypeId();
            $dbstatus = $this->db->is_exist($entity_object->id(),$entity,$entity_object->bundle());
            if(empty($dbstatus)){
              $fields = array(
                'entity_id'=> $entity_object->id() ,
                'entity_type' => $entity,
                'bundle' => $entity_object->bundle(),
                'label' => $entity_object->label(),
                'file' => $item['path']
              );
              $this->db->insert($fields);
            }
          }
        }
      }
    }
    return $items;


  }

  function get_export_path($entity) {
    if (is_object($entity)) {
      $entity_type = $entity->getEntityTypeId();
      $type = $entity->bundle();
      $config = \Drupal::config('content_export_yaml.contentexportsetting');
      $themes_str = $config->get('path_export_content_folder');
      if ($themes_str) {
        $themes_str = DRUPAL_ROOT . $themes_str;
        if ($type) {
          $final_path = $themes_str . '/' . $entity_type . '/' . $type;
        }
        else {
          $final_path = $themes_str . '/' . $entity_type;
        }
        return $final_path;
      }
      else {
        $this->logger->error('Path directory empty ');
        return FALSE;
      }
    }
  }

  public function redirectTo($url, $lang = NULL) {
    global $base_url;
    $path = $base_url . '/' . $url;
    $response = new RedirectResponse($path, 302);
    $response->send();
    return;
  }
}